@@ -41,24 +41,21 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <gnome.h>
 #include <libgnomeui/gnome-window-icon.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <gtk/gtkbuilder.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnomecanvas/gnome-canvas-pixbuf.h>
 #include <libgnome/libgnometypebuiltins.h>
 #include <gdk/gdkx.h>
+#include <gdk/gdk.h>
 
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 
 #include <gconf/gconf-client.h>
 
-#ifdef HAVE_NETWORKMANAGER
-#include <libnm_glib.h>
-#endif
-
 #include <libsoup/soup.h>
 
 #include <sys/types.h>
@@ -95,12 +92,6 @@ static const GOptionEntry options[] = {
 	{ NULL }
 };
 
-enum {
-	NETWORK_CONNECTED,
-	NETWORK_DISCONNECTED,
-	NETWORK_UNKNOWN
-};
-
 static void fill_stderr_info (GtkBuilder *ui);
 static void fill_custom_info (BugzillaApplication *app, GtkBuilder *ui);
 static void close_callback   (GtkWidget *widget, gpointer user_data);
@@ -366,8 +357,22 @@ static void
 link_callback (GtkLinkButton *button, gpointer user_data)
 {
 	const gchar *link = gtk_link_button_get_uri (button);
+#if GTK_CHECK_VERSION (2,13,0)
+	GdkAppLaunchContext *context;
 
-	if (gnome_vfs_url_show (link) != GNOME_VFS_OK) {
+	context = gdk_app_launch_context_new ();
+	gdk_app_launch_context_set_screen (context,
+					   gtk_widget_get_screen (GTK_WIDGET (button)));
+	gdk_app_launch_context_set_timestamp (context,
+					      gtk_get_current_event_time ());	
+
+	if (!g_app_info_launch_default_for_uri (link,
+						G_APP_LAUNCH_CONTEXT (context),
+						NULL))
+#else
+	if (!g_app_info_launch_default_for_uri (link, NULL, NULL))
+#endif
+    {
 		char *text;
 
 		text = g_markup_printf_escaped (_("Bug Buddy was unable to view the link \"%s\"\n"), link);
@@ -375,6 +380,10 @@ link_callback (GtkLinkButton *button, gpointer user_data)
 		g_free (text);
 	}
 
+#if GTK_CHECK_VERSION (2,13,0)
+	g_object_unref (context);
+#endif
+
 	return;
 }
 
@@ -1324,6 +1333,17 @@ bug_buddy_quit (GtkBuilder *ui)
 	gtk_main_quit ();
 }
 
+static gboolean
+keypress_callback (GtkWidget *widget, GdkEventKey *event, gpointer data)
+{
+	if (event->keyval == GDK_Escape) {
+		close_callback (NULL, data);
+		return TRUE;
+	}
+	/* let others handle the event */
+	return FALSE;
+}
+
 static void
 close_callback (GtkWidget *widget, gpointer user_data)
 {
@@ -1638,38 +1658,6 @@ fill_stderr_info (GtkBuilder *ui)
 	g_string_free (stderr_info, TRUE);
 }
 
-
-static GdkPixbuf*
-load_icon (const char *icon)
-{
-	GdkPixbuf *pixbuf = NULL;
-
-	if (g_path_is_absolute (icon)) {
-		pixbuf = gdk_pixbuf_new_from_file_at_size (icon, 48, 48, NULL);
-	}
-	if (pixbuf == NULL) {
-		pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-						   icon, 48, 0, NULL);
-	}
-	if (pixbuf == NULL && strrchr (icon, '.') != NULL) {
-		char *name;
-		name = g_strndup (icon, strlen (icon) - strlen (strrchr (icon, '.')));
-		pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-						   name, 48, 0, NULL);
-		g_free (name);
-	}
-
-	if (pixbuf == NULL) {
-		char *filename;
-		filename = g_strdup_printf (BUDDY_ICONDIR"/pixmaps/%s", icon);
-		pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 48, 48, NULL);
-		g_free (filename);
-	}
-
-	return pixbuf;
-}
-	
-
 int
 main (int argc, char *argv[])
 {
@@ -1728,6 +1716,7 @@ main (int argc, char *argv[])
 
 	main_window = GTK_WIDGET (gtk_builder_get_object (ui, "main-window"));
 	g_signal_connect (main_window, "delete-event", G_CALLBACK (delete_callback), ui);
+	g_signal_connect (main_window, "key-press-event", G_CALLBACK (keypress_callback), ui);
 
 	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (ui, "final-box")));
 
@@ -1785,13 +1774,9 @@ main (int argc, char *argv[])
 			g_object_set_data (G_OBJECT (ui), "app", app);
 	
 			if (app->icon) {
-				GdkPixbuf *pixbuf;
-				pixbuf = load_icon (app->icon);
-				if (pixbuf) {
-					gtk_image_set_from_pixbuf (GTK_IMAGE (gtk_builder_get_object (ui, "app-image")),
-			                           	   	   pixbuf);
-					g_object_unref(pixbuf);
-				}
+				gtk_image_set_from_icon_name (GTK_IMAGE (gtk_builder_get_object (ui, "app-image")),
+							      app->icon,
+							      GTK_ICON_SIZE_DIALOG);
 			}
 			fill_gnome_info (app, gnome_version, ui);
 		}
@@ -1853,14 +1838,9 @@ main (int argc, char *argv[])
 		g_object_set_data (G_OBJECT (ui), "app", app);
 	
 		if (app->icon) {
-			GdkPixbuf *pixbuf;
-			
-			pixbuf = load_icon (app->icon);
-			if (pixbuf) {
-				gtk_image_set_from_pixbuf (GTK_IMAGE (gtk_builder_get_object (ui, "app-image")),
-							   pixbuf);
-				g_object_unref (pixbuf);
-			}
+			gtk_image_set_from_icon_name (GTK_IMAGE (gtk_builder_get_object (ui, "app-image")),
+						      app->icon,
+						      GTK_ICON_SIZE_DIALOG);
 		}
 		fill_gnome_info (app, gnome_version, ui);
 		fill_custom_info (app, ui);
